package ru.dbotthepony.mc.otm.container;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.capabilities.Capability;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.function.Consumer;

public class MatteryContainer extends SimpleContainer {
	protected final Runnable watcher;

	public MatteryContainer(Runnable watcher, int i) {
		super(i);
		this.watcher = watcher;
	}

	public MatteryContainer of(@Nullable CompoundTag tag) {
		if (tag == null)
			return this;

		return MatteryContainer.of(watcher, tag, getContainerSize());
	}

	public MatteryContainer of(@Nullable Tag tag) {
		if (tag == null)
			return this;

		if (tag instanceof CompoundTag compound)
			return MatteryContainer.of(watcher, compound, getContainerSize());

		return this;
	}

	public MatteryContainer(Runnable watcher, ItemStack... p_19152_) {
		super(p_19152_);
		this.watcher = watcher;
	}

	public ContainerIteratorItemStack stackIterator() {
		return new ContainerIteratorItemStack(this);
	}

	public void consumeNonEmpty(Consumer<ItemStack> consumer) {
		stackIterator().consume(consumer);
	}

	public <T> ContainerIteratorCapability<T> capabilityIterator(Capability<T> cap) {
		return new ContainerIteratorCapability<>(this, cap);
	}

	public <T> void consumeCapability(Capability<T> cap, Consumer<T> consumer) {
		capabilityIterator(cap).consume(consumer);
	}

	@Override
	public void setChanged() {
		super.setChanged();
		watcher.run();
	}

	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();
		tag.putInt("size", getContainerSize());
		ListTag list = new ListTag();
		tag.put("items", list);

		for (int i = 0; i < getContainerSize(); i++)
			list.add(getItem(i).serializeNBT());

		return tag;
	}

	public static MatteryContainer of(Runnable watcher, @Nullable CompoundTag tag, int fallback_size) {
		if (tag == null)
			return new MatteryContainer(watcher, fallback_size);

		MatteryContainer container;

		if (tag.contains("size"))
			container = new MatteryContainer(watcher, tag.getInt("size"));
		else
			container = new MatteryContainer(watcher, fallback_size);

		if (tag.get("items") instanceof ListTag list)
			for (int i = 0; i < Math.min(list.size(), container.getContainerSize()); i++)
				if (list.get(i) instanceof CompoundTag get_tag)
					container.setItem(i, ItemStack.of(get_tag));

		return container;
	}

	public boolean hasEmptySlot() {
		for (int i = 0; i < getContainerSize(); i++) {
			if (getItem(i).isEmpty())
				return true;
		}

		return false;
	}
}
